#include <stdio.h>
#include <string.h>
#include <signal.h>

#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/ip.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/ip_icmp.h>

#include <sys/time.h>
#include <time.h>

#define BUFSIZE 1500

int finish = 0;

void sig_handler (int sig)
{
  if (sig == SIGINT)
    finish = 1;
}

int main (int argc, char *argv[])
{
  struct sigaction act;
  
  int sock, i;
  ssize_t length;
  char buffer[BUFSIZE];
  struct iphdr *ip;
  struct icmphdr *icmp;
  struct sockaddr_in address;
  socklen_t address_size;
  struct timeval tv;

  memset (&act, 0, sizeof(act));
  act.sa_handler = sig_handler;
  sigaction (SIGINT, &act, NULL);
  
  /* vytvorime raw socket */
  sock = socket (PF_INET, SOCK_RAW, IPPROTO_ICMP);

  if (sock == -1) {
    perror ("socket error");
    return -1;
  }

  address_size = sizeof(address);
  
  while (!finish) {
    /* prijmeme data */
    length = recvfrom (sock, buffer, BUFSIZE, 0, (struct sockaddr *)&address,
		       &address_size);
    
    if (length == -1) {
      if (errno != EINTR) {
	perror ("recvfrom error");
	close (sock);
	return -1;
      }
      else
	continue;
    }
    
    /* vypiseme cas */
    gettimeofday(&tv, NULL);
    printf ("%li.%03li\n", tv.tv_sec, tv.tv_usec);

    /* vypiseme data */
    ip = (struct iphdr *)buffer;
    icmp = (struct icmphdr *)&buffer[ip->ihl * 4];
    printf ("%s: ", inet_ntoa (address.sin_addr));
    for (i = ip->ihl * 4; i < length; i++)
      printf ("%02hhx ", buffer[i]);
    printf ("(%i bytes)\n", i);
  }
  
  /* zrusime socket */
  close (sock);
  
  return 0;
}
