#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <signal.h>

#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define BUFSIZE 1500

int finish = 0;

void sig_handler (int sig)
{
  if (sig == SIGINT)
    finish = 1;
}

int main (int argc, char *argv[])
{
  struct sigaction act;

  int sock, connsock;
  ssize_t length, count;
  char buffer[BUFSIZE];
  char *ip_addr;
  int port;
  struct sockaddr_in address, client_address;
  socklen_t client_address_size;
  
  if (argc == 3) {
    /* klient */
    
    ip_addr = strdup (argv[1]);
    port = atoi (argv[2]);
    
    /* vytvorime socket */
    sock = socket (PF_INET, SOCK_STREAM, 0);
    
    if (sock == -1) {
      perror ("socket error");
      return -1;
    }
    
    /* nastavime adresu serveru */
    address.sin_family = PF_INET;
    address.sin_port = htons (port);
    inet_aton (ip_addr, &(address.sin_addr));
    
    /* pripojime socket na adresu = navazeme spojeni */
    if (connect (sock, (struct sockaddr *)&address, sizeof(address)) == -1) {
      perror ("connect error");
      close (sock);
      return -1;
    }
    
    /* nacitame data */
    while ((count = fread (buffer, 1, BUFSIZE, stdin))) {
      /* odesleme data */
      length = send (sock, buffer, count, 0);
      
      if (length == -1) {
	perror ("send error");
        close (sock);
        return -1;
      }
    }
    
    /* zrusime socket */
    close (sock);
  } else if (argc == 2) {
    /* server */
  
    memset (&act, 0, sizeof(act));
    act.sa_handler = sig_handler;
    sigaction (SIGINT, &act, NULL);
    
    port = atoi (argv[1]);
    
    /* vytvorime socket */
    sock = socket (PF_INET, SOCK_STREAM, 0);
    
    if (sock == -1) {
      perror ("socket error");
      return -1;
    }
    
    /* nastavime lokalni adresu, na ktere budeme naslouchat
       (odkudkoliv na portu port) */
    address.sin_family = PF_INET;
    address.sin_port = htons (port);
    address.sin_addr.s_addr = INADDR_ANY;
    
    /* pripojime socket na adresu = pojmenujeme socket */
    if (bind (sock, (struct sockaddr *)&address, sizeof(address)) == -1) {
      perror ("bind error");
      close (sock);
      return -1;
    }
    
    /* a naslouchame na socketu, s frontou pozadavku na spojeni delky 10 */
    if (listen (sock, 10) == -1) {
      perror ("listen error");
      close (sock);
      return -1;
    }
    
    client_address_size = sizeof(client_address);
    
    do {
      /* prijmeme spojeni - vrati socket spojeni */
      connsock = accept (sock, (struct sockaddr *)&client_address,
			 &client_address_size);
      
      if (connsock == -1) {
	if (errno != EINTR) {
	  perror ("accept error");
	  close (sock);
	  return -1;
	}
	else
	  continue;
      }
      
      /* vypisujeme data */
      do {
	/* prijmeme data */
	length = recv (connsock, buffer, BUFSIZE, 0);
      
	if (length == -1) {
	  if (errno != EINTR)
	    perror ("recv error");
	  close (connsock);
	  break;
	}
	
	fwrite (buffer, 1, length, stdout);
      } while (length);
      
      /* zrusime socket spojeni = ukoncime spojeni */
      close (connsock);
    } while (!finish);
    
    /* zrusime socket */
    close (sock);
  }
  
  return 0;
}
